- test:
    definitions:
    - from: inline
      name: setup-networking
      path: inline/health-checks.yaml
      repository:
        metadata:
          format: Lava-Test Test Definition 1.0
          name: network-setup
        run:
          steps:
          - ip a || true
          - which dhclient || lava-test-raise "dhclient not found"
          - dhclient
    timeout:
      minutes: 10

- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository: https://gitlab.com/lava/hacking-session.git
      from: git
      lava-signal: kmsg
      path: hacking-session-tmate.yaml
      parameters:
        PUB_KEY: '{{ parameters.get("PUB_KEY", "") }}'
      name: hacking-session
