- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository: {{ test_definitions }}
{%if device.name != 'ssh-device' %}
      lava-signal: kmsg
{% endif %}
      from: url
      compression: zstd
      path: automated/linux/kselftest/kselftest.yaml
      parameters:
        TST_CMDFILES: "{{ cmdfile }}"
        KSELFTEST_PATH: "/opt/kselftests/default-in-kernel"
        SKIPFILE: "{{ parameters.get("SKIPFILE", "skipfile-lkft.yaml") }}"
        ENVIRONMENT: "production"
        SHARD_NUMBER: {{ parameters.get("SHARD_NUMBER", 1) }}
        SHARD_INDEX: {{ parameters.get("SHARD_INDEX", 1) }}
      name: {{ name }}

