- test:
    timeout:
      minutes: {{ timeout }}
    interactive:
    - name: purecap
      prompts: ["{{ '", "'.join(prompts)}}"]
      script:
      - command: "./morello-helloworld"
        name: helloworld
        successes:
        - message: "Hello from Morello!!"
      - command: "./morello-stack"
        name: stack
        successes:
        - message: "Test 1: Write inside the array: OK\\r\\nTest 2: Write beyond the array \\(expected SEGFAULT\\):Segmentation fault"
      - command: "echo 'arm' | ./morello-heap"
        name: heap
        successes:
        - message: "Input string: buffer: arm"
      - command: "echo '1234567890abcdef' | ./morello-heap"
        name: heap
        successes:
        - message: "Input string: Segmentation fault"
