- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository:
        metadata:
          format: lava-test test definition 1.0
          name: commands
          description: "Run commands"
        run:
          steps:
          - "lava-test-case {{ command_name }} --shell {{ commands }}"
      from: inline
      path: command
      name: commands
